@@ -1,55 +1,43 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 // Declarations for simple estimate of the memory being used by a program.
-// Not yet implemented for OS X*.
+// Not yet implemented for macOS*.
 // This header is an optional part of the test harness.
 // It assumes that "harness_assert.h" has already been included.
 
 #ifndef tbb_test_harness_allocator_H
 #define tbb_test_harness_allocator_H
 
+#include "harness_defs.h"
+
 #if __linux__ || __APPLE__ || __sun
 #include <unistd.h>
-#elif _WIN32 
+#elif _WIN32
 #include "tbb/machine/windows_api.h"
 #endif /* OS specific */
 #include <memory>
 #include <new>
-
-#if !TBB_USE_EXCEPTIONS && _MSC_VER
-    // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
-    #pragma warning (push)
-    #pragma warning (disable: 4530)
-#endif
-
+#include <cstdio>
 #include <stdexcept>
-
-#include <utility>    // for std::swap
-
-#if !TBB_USE_EXCEPTIONS && _MSC_VER
-    #pragma warning (pop)
-#endif
+#include <utility>
+#include __TBB_STD_SWAP_HEADER
 
 #include "tbb/atomic.h"
-#include "harness_defs.h"
+#include "tbb/tbb_allocator.h"
 
 #if __SUNPRO_CC
 using std::printf;
@@ -57,13 +45,20 @@ using std::printf;
 
 #if defined(_MSC_VER) && !defined(__INTEL_COMPILER)
     // Workaround for overzealous compiler warnings in /Wp64 mode
-    #pragma warning (push)
+    // #pragma warning (push)
 #if defined(_Wp64)
-    #pragma warning (disable: 4267)
+    // #pragma warning (disable: 4267)
+#endif
+#if _MSC_VER <= 1600
+    // #pragma warning (disable: 4355)
+#endif
+#if _MSC_VER <= 1800
+    // #pragma warning (disable: 4512)
 #endif
-    #pragma warning (disable: 4512)
 #endif
 
+#if TBB_INTERFACE_VERSION >= 7005
+// Allocator traits were introduced in 4.2 U5
 namespace Harness {
 #if __TBB_ALLOCATOR_TRAITS_PRESENT
     using std::true_type;
@@ -73,20 +68,23 @@ namespace Harness {
     using tbb::internal::false_type;
 #endif //__TBB_ALLOCATOR_TRAITS_PRESENT
 }
+#endif
 
 template<typename counter_type = size_t>
-struct arena_data  {
+struct arena_data {
     char * const my_buffer;
     size_t const my_size; //in bytes
     counter_type my_allocated; // in bytes
 
     template<typename T>
     arena_data(T * a_buffer, size_t a_size) __TBB_NOEXCEPT(true)
-    :   my_buffer(reinterpret_cast<char *>(a_buffer))
-    ,   my_size(a_size * sizeof(T) )
+    :   my_buffer(reinterpret_cast<char*>(a_buffer))
+    ,   my_size(a_size * sizeof(T))
     {
         my_allocated =0;
     }
+private:
+    void operator=( const arena_data& ); // NoAssign is not used to avoid dependency on harness.h
 };
 
 template<typename T, typename pocma = Harness::false_type, typename counter_type = size_t>
@@ -126,7 +124,7 @@ public:
     //! Allocate space for n objects, starting on a cache/sector line.
     pointer allocate( size_type n, const void* =0) {
         size_t new_size = (my_data->my_allocated += n*sizeof(T));
-        __TBB_ASSERT(my_data->my_allocated <= my_data->my_size,"trying to allocate more than was reserved");
+        ASSERT(my_data->my_allocated <= my_data->my_size,"trying to allocate more than was reserved");
         char* result =  &(my_data->my_buffer[new_size - n*sizeof(T)]);
         return reinterpret_cast<pointer>(result);
     }
@@ -134,11 +132,9 @@ public:
     //! Free block of memory that starts on a cache line
     void deallocate( pointer p_arg, size_type n) {
         char* p = reinterpret_cast<char*>(p_arg);
-        __TBB_ASSERT(p >=my_data->my_buffer && p <= my_data->my_buffer + my_data->my_size, "trying to deallocate pointer not from arena ?");
-        __TBB_ASSERT(p + n*sizeof(T) <= my_data->my_buffer + my_data->my_size, "trying to deallocate incorrect number of items?");
-        tbb::internal::suppress_unused_warning(p_arg);
-        tbb::internal::suppress_unused_warning(p);
-        tbb::internal::suppress_unused_warning(n);
+        ASSERT(p >=my_data->my_buffer && p <= my_data->my_buffer + my_data->my_size, "trying to deallocate pointer not from arena ?");
+        ASSERT(p + n*sizeof(T) <= my_data->my_buffer + my_data->my_size, "trying to deallocate incorrect number of items?");
+        tbb::internal::suppress_unused_warning(p, n);
     }
 
     //! Largest value for which method allocate might succeed.
@@ -161,9 +157,8 @@ public:
     //! Destroy value at location pointed to by p.
     void destroy( pointer p ) {
         p->~value_type();
-#if _MSC_VER == 1800
+        // suppress "unreferenced parameter" warnings by MSVC up to and including 2015
         tbb::internal::suppress_unused_warning(p);
-#endif
     }
 
     friend bool operator==(arena const& lhs, arena const& rhs){
@@ -220,7 +215,7 @@ public:
     static_counting_allocator(const base_alloc_t& src) throw()
     : base_alloc_t(src) { }
 
-    static_counting_allocator(const static_counting_allocator& src) throw() 
+    static_counting_allocator(const static_counting_allocator& src) throw()
     : base_alloc_t(src) { }
 
     template<typename U, typename C>
@@ -236,9 +231,10 @@ public:
                 __TBB_THROW( std::bad_alloc() );
             return NULL;
         }
+        pointer p = base_alloc_t::allocate(n, pointer(0));
         allocations++;
         items_allocated += n;
-        return base_alloc_t::allocate(n, pointer(0));
+        return p;
     }
 
     pointer allocate(const size_type n, const void * const)
@@ -472,9 +468,10 @@ public:
     {
         if(max_items && items_allocated + n >= max_items)
             __TBB_THROW( std::bad_alloc() );
+        pointer p = base_alloc_t::allocate(n, pointer(0));
         ++allocations;
         items_allocated += n;
-        return base_alloc_t::allocate(n, pointer(0));
+        return p;
     }
 
     pointer allocate(const size_type n, const void * const)
@@ -515,14 +512,14 @@ public:
 
     pointer allocate(const size_type n, const void *hint = 0 ) {
         pointer ptr = base_allocator_type::allocate( n, hint );
-        std::memset( ptr, 0xE3E3E3E3, n * sizeof(value_type) );
+        std::memset( (void*)ptr, 0xE3E3E3E3, n * sizeof(value_type) );
         return ptr;
     }
 };
 
 //! Analogous to std::allocator<void>, as defined in ISO C++ Standard, Section 20.4.1
 /** @ingroup memory_allocation */
-template<template<typename T> class Allocator> 
+template<template<typename T> class Allocator>
 class debug_allocator<void, Allocator> : public Allocator<void> {
 public:
     typedef Allocator<void> base_allocator_type;
@@ -579,10 +576,262 @@ public:
 
 };
 
-#if defined(_MSC_VER)
-    // Workaround for overzealous compiler warnings in /Wp64 mode
-    #pragma warning (pop)
-#endif // warning 4267,4512 is back
+template <typename T>
+class pmr_stateful_allocator
+{
+private:
+    pmr_stateful_allocator& operator=(const pmr_stateful_allocator&); /* = deleted */
+public:
+    typedef T value_type;
+    typedef Harness::false_type propagate_on_container_move_assignment;
+    typedef Harness::false_type propagate_on_container_copy_assignment;
+    typedef Harness::false_type propagate_on_container_swap;
+
+// These types are required in C++03
+#if !__TBB_ALLOCATOR_TRAITS_PRESENT
+    typedef value_type* pointer;
+    typedef const value_type* const_pointer;
+    typedef value_type& reference;
+    typedef const value_type& const_reference;
+    typedef size_t size_type;
+    typedef ptrdiff_t difference_type;
+    template<class U> struct rebind {
+        typedef pmr_stateful_allocator<U> other;
+    };
+#endif
+
+    pmr_stateful_allocator() throw() : unique_pointer(this) {}
+
+    pmr_stateful_allocator(const pmr_stateful_allocator &a) : unique_pointer(a.unique_pointer) {}
+
+    template<typename U>
+    pmr_stateful_allocator(const pmr_stateful_allocator<U> &a) throw() : unique_pointer(a.unique_pointer) {}
+
+    value_type* allocate( size_t n, const void* /*hint*/ = 0 ) {
+        return static_cast<value_type*>( malloc( n * sizeof(value_type) ) );
+    }
+
+    void deallocate( value_type* p, size_t ) {
+        free( p );
+    }
+
+#if __TBB_ALLOCATOR_CONSTRUCT_VARIADIC
+    //! Copy-construct value at location pointed to by p.
+    template<typename U, typename... Args>
+    void construct(U *p, Args&&... args)
+    {
+        ::new((void *)p) U(std::forward<Args>(args)...);
+    }
+#else // __TBB_ALLOCATOR_CONSTRUCT_VARIADIC
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    void construct(value_type* p, value_type&& value) { ::new((void*)(p)) value_type(std::move(value)); }
+#endif
+    void construct(value_type* p, const value_type& value) { ::new((void*)(p)) value_type(value); }
+#endif // __TBB_ALLOCATOR_CONSTRUCT_VARIADIC
+
+    //! Destroy value at location pointed to by p.
+    void destroy(value_type* p) {
+        p->~value_type();
+        // suppress "unreferenced parameter" warnings by MSVC up to and including 2015
+        tbb::internal::suppress_unused_warning(p);
+    }
+
+    friend bool operator==(pmr_stateful_allocator const& lhs, pmr_stateful_allocator const& rhs){
+        return lhs.unique_pointer == rhs.unique_pointer;
+    }
+
+    friend bool operator!=(pmr_stateful_allocator const& rhs, pmr_stateful_allocator const& lhs){
+        return !(lhs == rhs);
+    }
+
+    void* unique_pointer;
+};
+
+// C++03 allocator doesn't have to be assignable or swappable, so
+// tbb::internal::allocator_traits defines POCCA and POCS as false_type
+#if __TBB_ALLOCATOR_TRAITS_PRESENT
+#include "tbb/internal/_allocator_traits.h" // Need traits_true/false_type
+
+template <typename Allocator, typename POCMA = tbb::internal::traits_false_type,
+          typename POCCA = tbb::internal::traits_false_type, typename POCS = tbb::internal::traits_false_type>
+struct propagating_allocator : Allocator {
+    typedef POCMA propagate_on_container_move_assignment;
+    typedef POCCA propagate_on_container_copy_assignment;
+    typedef POCS propagate_on_container_swap;
+    bool* propagated_on_copy_assignment;
+    bool* propagated_on_move_assignment;
+    bool* propagated_on_swap;
+    bool* selected_on_copy_construction;
+
+    template <typename U>
+    struct rebind {
+        typedef propagating_allocator<typename tbb::internal::allocator_rebind<Allocator, U>::type,
+                                      POCMA, POCCA, POCS> other;
+    };
+
+    propagating_allocator() : propagated_on_copy_assignment(NULL),
+                              propagated_on_move_assignment(NULL),
+                              propagated_on_swap(NULL),
+                              selected_on_copy_construction(NULL) {}
+
+    propagating_allocator(bool& poca, bool& poma, bool& pos, bool& soc)
+        : propagated_on_copy_assignment(&poca),
+          propagated_on_move_assignment(&poma),
+          propagated_on_swap(&pos),
+          selected_on_copy_construction(&soc) {}
+
+    propagating_allocator(const propagating_allocator& other)
+        : Allocator(other),
+          propagated_on_copy_assignment(other.propagated_on_copy_assignment),
+          propagated_on_move_assignment(other.propagated_on_move_assignment),
+          propagated_on_swap(other.propagated_on_swap),
+          selected_on_copy_construction(other.selected_on_copy_construction) {}
+
+    template <typename Allocator2>
+    propagating_allocator(const propagating_allocator<Allocator2, POCMA, POCCA, POCS>& other)
+        : Allocator(other),
+          propagated_on_copy_assignment(other.propagated_on_copy_assignment),
+          propagated_on_move_assignment(other.propagated_on_move_assignment),
+          propagated_on_swap(other.propagated_on_swap),
+          selected_on_copy_construction(other.selected_on_copy_construction) {}
+
+    propagating_allocator& operator=(const propagating_allocator&) {
+        ASSERT(POCCA::value, "Allocator should not copy assign if pocca is false");
+        if (propagated_on_copy_assignment)
+            *propagated_on_copy_assignment = true;
+        return *this;
+    }
+
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    propagating_allocator& operator=(propagating_allocator&&) {
+        ASSERT(POCMA::value, "Allocator should not move assign if pocma is false");
+        if (propagated_on_move_assignment)
+            *propagated_on_move_assignment = true;
+        return *this;
+    }
+#endif
+
+    propagating_allocator select_on_container_copy_construction() const {
+        if (selected_on_copy_construction)
+            *selected_on_copy_construction = true;
+        return *this;
+    }
+};
+
+namespace propagating_allocators {
+typedef tbb::tbb_allocator<int> base_allocator;
+typedef tbb::internal::traits_true_type true_type;
+typedef tbb::internal::traits_false_type false_type;
+
+typedef propagating_allocator<base_allocator, /*POCMA=*/true_type, /*POCCA=*/true_type,
+                              /*POCS=*/true_type> always_propagating_allocator;
+typedef propagating_allocator<base_allocator, false_type, false_type, false_type> never_propagating_allocator;
+typedef propagating_allocator<base_allocator, true_type, false_type, false_type> pocma_allocator;
+typedef propagating_allocator<base_allocator, false_type, true_type, false_type> pocca_allocator;
+typedef propagating_allocator<base_allocator, false_type, false_type, true_type> pocs_allocator;
+}
+
+template <typename Allocator, typename POCMA, typename POCCA, typename POCS>
+void swap(propagating_allocator<Allocator, POCMA, POCCA, POCS>& lhs,
+          propagating_allocator<Allocator, POCMA, POCCA, POCS>&) {
+    ASSERT(POCS::value, "Allocator should not swap if pocs is false");
+    if (lhs.propagated_on_swap)
+        *lhs.propagated_on_swap = true;
+}
+
+template <typename ContainerType>
+void test_allocator_traits_support() {
+    typedef typename ContainerType::allocator_type allocator_type;
+    typedef std::allocator_traits<allocator_type> allocator_traits;
+    typedef typename allocator_traits::propagate_on_container_copy_assignment pocca_type;
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    typedef typename allocator_traits::propagate_on_container_move_assignment pocma_type;
+#endif
+    typedef typename allocator_traits::propagate_on_container_swap pocs_type;
+
+    bool propagated_on_copy = false;
+    bool propagated_on_move = false;
+    bool propagated_on_swap = false;
+    bool selected_on_copy = false;
+
+    allocator_type alloc(propagated_on_copy, propagated_on_move, propagated_on_swap, selected_on_copy);
+
+    ContainerType c1(alloc), c2(c1);
+    ASSERT(selected_on_copy, "select_on_container_copy_construction function was not called");
+
+    c1 = c2;
+    ASSERT(propagated_on_copy == pocca_type::value, "Unexpected allocator propagation on copy assignment");
+
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    c2 = std::move(c1);
+    ASSERT(propagated_on_move == pocma_type::value, "Unexpected allocator propagation on move assignment");
+#endif
+
+    c1.swap(c2);
+    ASSERT(propagated_on_swap == pocs_type::value, "Unexpected allocator propagation on swap");
+}
+
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+class non_movable_object {
+    non_movable_object() {}
+private:
+    non_movable_object(non_movable_object&&);
+    non_movable_object& operator=(non_movable_object&&);
+};
+
+template <typename ContainerType>
+void test_allocator_traits_with_non_movable_value_type() {
+    // Check, that if pocma is true, container allows move assignment without per-element move
+    typedef typename ContainerType::allocator_type allocator_type;
+    typedef std::allocator_traits<allocator_type> allocator_traits;
+    typedef typename allocator_traits::propagate_on_container_move_assignment pocma_type;
+    ASSERT(pocma_type::value, "Allocator POCMA must be true for this test");
+    allocator_type alloc;
+    ContainerType container1(alloc), container2(alloc);
+    container1 = std::move(container2);
+}
+#endif // __TBB_CPP11_RVALUE_REF_PRESENT
+
+#endif // __TBB_ALLOCATOR_TRAITS_PRESENT
+
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+
+template<typename Allocator>
+class allocator_aware_data {
+public:
+    static bool assert_on_constructions;
+    typedef Allocator allocator_type;
+
+    allocator_aware_data(const allocator_type& allocator = allocator_type())
+        : my_allocator(allocator), my_value(0) {}
+    allocator_aware_data(int v, const allocator_type& allocator = allocator_type())
+        : my_allocator(allocator), my_value(v) {}
+    allocator_aware_data(const allocator_aware_data&) {
+        ASSERT(!assert_on_constructions, "Allocator should propagate to the data during copy construction");
+    }
+    allocator_aware_data(allocator_aware_data&&) {
+        ASSERT(!assert_on_constructions, "Allocator should propagate to the data during move construction");
+    }
+    allocator_aware_data(const allocator_aware_data& rhs, const allocator_type& allocator)
+        : my_allocator(allocator), my_value(rhs.my_value) {}
+    allocator_aware_data(allocator_aware_data&& rhs, const allocator_type& allocator)
+        : my_allocator(allocator), my_value(rhs.my_value) {}
+
+    int value() const { return my_value; }
+private:
+    allocator_type my_allocator;
+    int my_value;
+};
+
+template<typename Allocator>
+bool allocator_aware_data<Allocator>::assert_on_constructions = false;
+
+#endif // __TBB_CPP11_RVALUE_REF_PRESENT
+
+#if defined(_MSC_VER) && !defined(__INTEL_COMPILER)
+    // Workaround for overzealous compiler warnings
+    // #pragma warning (pop)
+#endif // warning 4267,4512,4355 is back
 
 namespace Harness {
 
@@ -593,6 +842,11 @@ namespace Harness {
             // Compare real pointers.
             return t1.lock().get() == t2.lock().get();
         }
+        template <typename T>
+        static bool compare( const std::unique_ptr<T> &t1, const std::unique_ptr<T> &t2 ) {
+            // Compare real values.
+            return *t1 == *t2;
+        }
         template <typename T1, typename T2>
         static bool compare( const std::pair< const std::weak_ptr<T1>, std::weak_ptr<T2> > &t1,
                 const std::pair< const std::weak_ptr<T1>, std::weak_ptr<T2> > &t2 ) {
