@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "harness_defs.h"
@@ -43,18 +39,13 @@ using std::memcmp;
 #if _MSC_VER && !defined(__INTEL_COMPILER)
     // Unary minus operator applied to unsigned type, result still unsigned
     // Constant conditional expression
-    #pragma warning( disable: 4127 4310 )
+    // #pragma warning( disable: 4127 4310 )
 #endif
 
 #if __TBB_GCC_STRICT_ALIASING_BROKEN
-//    #pragma GCC diagnostic ignored "-Wstrict-aliasing"
+    // #pragma GCC diagnostic ignored "-Wstrict-aliasing"
 #endif
 
-// Intel(R) Compiler have an issue when a scoped enum with a specified underlying type has negative values.
-#define __TBB_ICC_SCOPED_ENUM_WITH_UNDERLYING_TYPE_NEGATIVE_VALUE_BROKEN ( _MSC_VER && !__TBB_DEBUG && __INTEL_COMPILER && __INTEL_COMPILER <= 1500 )
-// Intel(R) Compiler have an issue with __atomic_load_explicit from a scoped enum with a specified underlying type.
-#define __TBB_ICC_SCOPED_ENUM_WITH_UNDERLYING_TYPE_ATOMIC_LOAD_BROKEN ( TBB_USE_ICC_BUILTINS && !__TBB_DEBUG && __INTEL_COMPILER && __INTEL_COMPILER <= 1500 )
-
 enum LoadStoreExpression {
     UseOperators,
     UseImplicitAcqRel,
@@ -102,7 +93,7 @@ template<typename T, LoadStoreExpression E> tbb::atomic<T> TestStruct<T, E>::gCo
 //! Test compare_and_swap template members of class atomic<T> for memory_semantics=M
 template<typename T,tbb::memory_semantics M>
 void TestCompareAndSwapWithExplicitOrdering( T i, T j, T k ) {
-    ASSERT( i!=k, "values must be distinct" ); 
+    ASSERT( i!=k && i!=j, "values must be distinct" );
     // Test compare_and_swap that should fail
     TestStruct<T> x(i);
     T old = x.counter.template compare_and_swap<M>( j, k );
@@ -117,7 +108,7 @@ void TestCompareAndSwapWithExplicitOrdering( T i, T j, T k ) {
 //! i, j, k must be different values
 template<typename T>
 void TestCompareAndSwap( T i, T j, T k ) {
-    ASSERT( i!=k, "values must be distinct" );
+    ASSERT( i!=k && i!=j, "values must be distinct" );
     // Test compare_and_swap that should fail
     TestStruct<T> x(i);
     T old = x.counter.compare_and_swap( j, k );
@@ -174,7 +165,7 @@ void TestFetchAndStore( T i, T j ) {
 #if _MSC_VER && !defined(__INTEL_COMPILER)
     // conversion from <bigger integer> to <smaller integer>, possible loss of data
     // the warning seems a complete nonsense when issued for e.g. short+=short
-    #pragma warning( disable: 4244 )
+    // #pragma warning( disable: 4244 )
 #endif
 
 //! Test fetch_and_add members of class atomic<T> for memory_semantics=M
@@ -303,7 +294,8 @@ namespace initialization_tests {
         tbb::aligned_space<atomic_t> non_zeroed_storage;
         enum {fill_value = 0xFF };
         test_initialization_fixture(){
-            memset(non_zeroed_storage.begin(),fill_value,sizeof(non_zeroed_storage));
+            memset(static_cast<void*>(non_zeroed_storage.begin()),fill_value,
+                   sizeof(non_zeroed_storage));
             ASSERT( char(fill_value)==*(reinterpret_cast<char*>(non_zeroed_storage.begin()))
                     ,"failed to fill the storage; memset error?");
         }
@@ -376,7 +368,7 @@ namespace test_constexpr_initialization_helper {
     struct white_box_ad_hoc_type {
         int _int;
         constexpr white_box_ad_hoc_type(int a =0) : _int(a) {};
-        constexpr operator int() const { return _int;}
+        constexpr operator int() const { return _int; }
     };
 }
 //some white boxing
@@ -385,7 +377,7 @@ namespace tbb { namespace internal {
     struct atomic_impl<test_constexpr_initialization_helper::white_box_ad_hoc_type>: atomic_impl<int> {
         atomic_impl() = default;
         constexpr atomic_impl(test_constexpr_initialization_helper::white_box_ad_hoc_type value):atomic_impl<int>(value){}
-        constexpr operator int(){ return this->my_storage.my_value;}
+        constexpr operator int() const { return this->my_storage.my_value; }
     };
 }}
 
@@ -396,6 +388,7 @@ void TestConstExprInitializationIsTranslationTime(){
     constexpr atomic_t a(8);
     ASSERT(a == 8,ct_init_failed_msg);
 
+#if !__TBB_CONSTEXPR_MEMBER_FUNCTION_BROKEN
     constexpr tbb::atomic<test_constexpr_initialization_helper::white_box_ad_hoc_type> ct_atomic(10);
     //for some unknown reason clang does not managed to enum syntax
 #if __clang__
@@ -407,6 +400,7 @@ void TestConstExprInitializationIsTranslationTime(){
     ASSERT(ct_atomic_value_ten == 10,ct_init_failed_msg);
     int array[ct_atomic_value_ten];
     ASSERT(Harness::array_length(array) == 10,ct_init_failed_msg);
+#endif //__TBB_CONSTEXPR_MEMBER_FUNCTION_BROKEN
 }
 
 #include <string>
@@ -475,7 +469,7 @@ namespace TestConstExprInitializationOfGlobalObjectsHelper{
        static_before(){ result = (static_atomic==ct_value); }            \
     } ;                                                                  \
                                                                          \
-    typename tester<T>::static_before tester<T>::static_before_;         \
+    tester<T>::static_before tester<T>::static_before_;                  \
     tbb::atomic<T> tester<T>::static_atomic(ct_value);                   \
                                                                          \
     auto_registered_tests_helper::registration<T> tester<T>::registered; \
@@ -489,11 +483,15 @@ namespace TestConstExprInitializationOfGlobalObjectsHelper{
     }
 
     void CallExprInitTests(){
+#   if __TBB_STATIC_CONSTEXPR_INIT_BROKEN
+        REPORT("Known issue: Compile-time initialization fails for static tbb::atomic variables\n");
+#   else
         using namespace auto_registered_tests_helper;
         for (size_t i =0; i<const_expr_tests.size(); ++i){
             (*const_expr_tests[i])();
         }
-        REMARK("ran  %d consrexpr static init test \n",const_expr_tests.size());
+        REMARK("ran %d constexpr static init test \n",const_expr_tests.size());
+#   endif
     }
 
     //TODO: unify somehow list of tested types with one in TestMain
@@ -612,8 +610,8 @@ void TestAlignment( const char* name ) {
 }
 
 #if _MSC_VER && !defined(__INTEL_COMPILER)
-    // unary minus operator applied to unsigned type, result still unsigned
-    #pragma warning( disable: 4146 )
+    // #pragma warning( disable: 4146 ) // unary minus operator applied to unsigned type, result still unsigned
+    // #pragma warning( disable: 4334 ) // result of 32-bit shift implicitly converted to 64 bits
 #endif
 
 /** T is an integral type. */
@@ -621,11 +619,12 @@ template<typename T>
 void TestAtomicInteger( const char* name ) {
     REMARK("testing atomic<%s> (size=%d)\n",name,sizeof(tbb::atomic<T>));
     TestAlignment<T>(name);
-    TestOperations<T>(0L,T(-T(1)),T(1));
+    TestOperations<T>(0L, T(-T(1)), T(1));
     for( int k=0; k<int(sizeof(long))*8-1; ++k ) {
-        TestOperations<T>(T(1L<<k),T(~(1L<<k)),T(1-(1L<<k)));
-        TestOperations<T>(T(-1L<<k),T(~(-1L<<k)),T(1-(-1L<<k)));
-        TestFetchAndAdd<T>(T(-1L<<k));
+        const long p = 1L<<k;
+        TestOperations<T>(T(p), T(~(p)), T(1-(p)));
+        TestOperations<T>(T(-(p)), T(~(-(p))), T(1-(-(p))));
+        TestFetchAndAdd<T>(T(-(p)));
     }
     TestParallel<T>( name );
 }
@@ -686,8 +685,8 @@ void TestAtomicPointerToTypeOfUnknownSize( const char* name ) {
 
 void TestAtomicBool() {
     REMARK("testing atomic<bool>\n");
-    TestOperations<bool>(true,true,false);
-    TestOperations<bool>(false,false,true);
+    TestOperations<bool>(false,true,true);
+    TestOperations<bool>(true,false,false);
     TestParallel<bool>( "bool" );
 }
 
@@ -716,9 +715,9 @@ void TestAtomicEnum() {
 enum class ScopedColor1 {ScopedRed,ScopedGreen,ScopedBlue=-1};
 // TODO: extend the test to cover 2 byte scoped enum as well
 #if __TBB_ICC_SCOPED_ENUM_WITH_UNDERLYING_TYPE_NEGATIVE_VALUE_BROKEN
-enum class ScopedColor2 : char {ScopedZero, ScopedOne,ScopedRed=42,ScopedGreen=-1,ScopedBlue=127};
+enum class ScopedColor2 : signed char {ScopedZero, ScopedOne,ScopedRed=42,ScopedGreen=-1,ScopedBlue=127};
 #else
-enum class ScopedColor2 : char {ScopedZero, ScopedOne,ScopedRed=-128,ScopedGreen=-1,ScopedBlue=127};
+enum class ScopedColor2 : signed char {ScopedZero, ScopedOne,ScopedRed=-128,ScopedGreen=-1,ScopedBlue=127};
 #endif
 
 // TODO: replace the hack of getting symbolic enum name with a better implementation
@@ -729,7 +728,7 @@ std::string to_string<ScopedColor1>(const ScopedColor1& a){
 }
 template<>
 std::string to_string<ScopedColor2>(const ScopedColor2& a){
-    return enum_strings[a==ScopedColor2::ScopedRed? 2 : 
+    return enum_strings[a==ScopedColor2::ScopedRed? 2 :
         a==ScopedColor2::ScopedGreen? 3 : a==ScopedColor2::ScopedBlue? 4 : (int)a ];
 }
 
@@ -1052,7 +1051,7 @@ int TestMain () {
 #   elif __TBB_CAS_8_CODEGEN_BROKEN
          REPORT("Known issue: compiler generates incorrect code for 64-bit atomics on this configuration\n");
 #   else
-         REPORT("64-bit atomics not supported\n");
+         REPORT("Known issue: 64-bit atomics are not supported\n");
          ASSERT(sizeof(long long)==8, "type long long is not 64 bits");
 #   endif
     TestAtomicInteger<unsigned long>("unsigned long");
@@ -1129,8 +1128,7 @@ public:
         ASSERT( raw_space<=reinterpret_cast<char*>(y), "y starts before raw_space" );
         ASSERT( reinterpret_cast<char*>(y+1) <= raw_space+sizeof(raw_space), "y starts after raw_space" );
         ASSERT( !(aligned ^ tbb::internal::is_aligned(y,sizeof(T))), "y is not aligned as it required" );
-        new (y) tbb::atomic<T> ();
-        return *y;
+        return *(new (y) tbb::atomic<T>());
     }
 };
 
@@ -1143,7 +1141,7 @@ struct FlagAndMessage: AlignedAtomic<T,aligned> {
     //! Non-zero if message is ready
     T message;
     FlagAndMessage(): flag(FlagAndMessage::construct_atomic()) {
-        std::memset(pad,0,sizeof(pad));
+        std::memset(static_cast<void*>(pad),0,sizeof(pad));
     }
 };
 
@@ -1285,7 +1283,7 @@ public:
                     if( flag!=(T)-1 ) {
                         REPORT("ERROR: flag!=(T)-1 k=%d i=%d trial=%x type=%s (atomicity problem?)\n", k, i, trial, name );
                         ParallelError = true;
-                    } 
+                    }
                     if( !IsRelaxed(E) && message!=(T)-1 ) {
                         REPORT("ERROR: message!=(T)-1 k=%d i=%d trial=%x type=%s mode=%d (memory fence problem?)\n", k, i, trial, name, E );
                         ParallelError = true;
@@ -1395,12 +1393,12 @@ public:
 
 #if _MSC_VER==1500 && !defined(__INTEL_COMPILER)
     // VS2008/VC9 seems to have an issue; limits pull in math.h
-    #pragma warning( push )
-    #pragma warning( disable: 4985 )
+    // #pragma warning( push )
+    // #pragma warning( disable: 4985 )
 #endif
 #include <limits> /* Need std::numeric_limits */
 #if _MSC_VER==1500 && !defined(__INTEL_COMPILER)
-    #pragma warning( pop )
+    // #pragma warning( pop )
 #endif
 
 //! Commonality inherited by specializations for floating-point types.
@@ -1493,7 +1491,7 @@ public:
 template<typename T> void TestAssignmentSignature( T& (T::*)(const T&) ) {}
 
 #if _MSC_VER && !defined(__INTEL_COMPILER)
-    #pragma warning( disable: 4355 4800 )
+    // #pragma warning( disable: 4355 4800 )
 #endif
 
 template<typename T, bool aligned>
