@@ -1,21 +1,17 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 /** This test checks the automatic propagation of master thread FPU settings
@@ -186,7 +182,7 @@ class TestContextFpuEnvTask : public tbb::task {
 #endif
 public:
     TestContextFpuEnvTask( int _arenaNum, int _mode, int _depth = 0 ) : arenaNum(_arenaNum), mode(_mode), depth(_depth) {}
-    tbb::task* execute() {
+    tbb::task* execute() __TBB_override {
         AssertMode( mode );
         if ( depth < MAX_DEPTH ) {
             // Test default context.
@@ -248,13 +244,13 @@ void TestContextFpuEnvBody::operator()( const tbb::blocked_range<int> &r ) const
     int end = r.end();
     if ( end-1 == numModes ) {
         // For a default context our mode should be inherited.
-        tbb::task::spawn_root_and_wait( 
+        tbb::task::spawn_root_and_wait(
             *new( tbb::task::allocate_root() ) TestContextFpuEnvTask( arenaNum, mode, depth ) );
         AssertMode( newMode );
         end--;
     }
     for ( int i=r.begin(); i<end; ++i ) {
-        tbb::task::spawn_root_and_wait( 
+        tbb::task::spawn_root_and_wait(
             *new( tbb::task::allocate_root(*contexts[i]) ) TestContextFpuEnvTask( arenaNum, i, depth ) );
         AssertMode( newMode );
     }
@@ -346,7 +342,7 @@ tbb::task_group_context glbIsolatedCtx( tbb::task_group_context::isolated );
 int glbIsolatedCtxMode = -1;
 
 struct TestGlobalIsolatedContextTask : public tbb::task {
-    tbb::task* execute() {
+    tbb::task* execute() __TBB_override {
         AssertFPMode( glbIsolatedCtxMode );
         return NULL;
     }
